@@ -20,10 +20,10 @@
 
 """Python module for handling PSKC files
 
-This Python library handles Portable Symmetric Key Container (PSKC) files
-as defined in RFC6030. PSKC files are used to transport and provision
-symmetric keys (seed files) to different types of crypto modules, commonly
-one-time password tokens or other authentication devices.
+This Python library handles Portable Symmetric Key Container (PSKC) files as
+defined in RFC 6030. PSKC files are used to transport and provision symmetric
+keys (seed files) to different types of crypto modules, commonly one-time
+password tokens or other authentication devices.
 
 The main goal of this module is to be able to extract keys from PSKC files
 for use in an OTP authentication system.
@@ -37,9 +37,9 @@ The following prints all keys, decrypting using a password:
 ...     print key.serial, key.secret
 987654321 12345678901234567890
 
-The module should be able to handle most common PSKC files. Checking
-embedded signatures, asymmetric keys and writing files are on the wishlist
-(patches welcome).
+The module should be able to handle most common PSKC files. Checking embedded
+signatures, asymmetric keys and writing files are on the wishlist (patches
+welcome).
 """
 
 
@@ -47,7 +47,7 @@ __all__ = ['PSKC', '__version__']
 
 
 # the version number of the library
-__version__ = '0.1'
+__version__ = '0.2'
 
 
 class PSKC(object):
@@ -62,33 +62,57 @@ class PSKC(object):
       keys: list of keys (Key instances)
     """
 
-    def __init__(self, filename):
-        from xml.etree import ElementTree
+    def __init__(self, filename=None):
         from pskc.encryption import Encryption
+        from pskc.exceptions import ParseError
         from pskc.mac import MAC
         self.version = None
         self.id = None
         self.encryption = Encryption()
         self.mac = MAC(self)
         self.keys = []
-        tree = ElementTree.parse(filename)
-        self.parse(tree.getroot())
+        if filename is not None:
+            from pskc.parse import etree
+            try:
+                tree = etree.parse(filename)
+            except Exception:
+                raise ParseError('Error parsing XML')
+            self.parse(tree.getroot())
+        else:
+            self.version = '1.0'
 
     def parse(self, container):
         """Read information from the provided <KeyContainer> tree."""
-        from pskc.parse import namespaces
+        from pskc.exceptions import ParseError
         from pskc.key import Key
+        from pskc.parse import find, findall
+        if not container.tag.endswith('KeyContainer'):
+            raise ParseError('Missing KeyContainer')
         # the version of the PSKC schema
-        self.version = container.attrib.get('Version')
+        self.version = container.get('Version')
+        if self.version != '1.0':
+            raise ParseError('Unsupported version %r' % self.version)
         # unique identifier for the container
-        self.id = container.attrib.get('Id')
+        self.id = container.get('Id')
         # handle EncryptionKey entries
-        self.encryption.parse(container.find(
-            'pskc:EncryptionKey', namespaces=namespaces))
+        self.encryption.parse(find(container, 'pskc:EncryptionKey'))
         # handle MACMethod entries
-        self.mac.parse(container.find(
-            'pskc:MACMethod', namespaces=namespaces))
+        self.mac.parse(find(container, 'pskc:MACMethod'))
         # handle KeyPackage entries
-        for package in container.findall(
-                'pskc:KeyPackage', namespaces=namespaces):
-            self.keys.append(Key(self, package))
+        for key_package in findall(container, 'pskc:KeyPackage'):
+            self.keys.append(Key(self, key_package))
+
+    def add_key(self, **kwargs):
+        """Create a new key instance for the PSKC file.
+
+        The new key is initialised with properties from the provided keyword
+        arguments if any."""
+        from pskc.key import Key
+        key = Key(self)
+        self.keys.append(key)
+        # assign the kwargs as key properties
+        for k, v in kwargs.items():
+            if not hasattr(key, k):
+                raise AttributeError()
+            setattr(key, k, v)
+        return key
