@@ -37,7 +37,6 @@ class DataType(object):
       plain_value: raw unencrypted value if present (possibly base64 encoded)
       encrypted_value: reference to an EncryptedValue instance
       value_mac: reference to a ValueMAC instance
-      value: the plaintext value (decrypted if necessary)
     """
 
     def __init__(self, key, element=None):
@@ -52,14 +51,17 @@ class DataType(object):
         The element is expected to contain <PlainValue>, <EncryptedValue>
         and/or ValueMAC elements that contain information on the actual
         value."""
-        from pskc.parse import g_e_v, namespaces
+        from pskc.parse import find, findtext
         if element is None:
             return
-        self.plain_value = g_e_v(element, 'pskc:PlainValue')
-        self.encrypted_value.parse(element.find(
-            'pskc:EncryptedValue', namespaces=namespaces))
-        self.value_mac.parse(element.find(
-            'pskc:ValueMAC', namespaces=namespaces))
+        self.plain_value = findtext(element, 'pskc:PlainValue')
+        self.encrypted_value.parse(find(element, 'pskc:EncryptedValue'))
+        self.value_mac.parse(find(element, 'pskc:ValueMAC'))
+
+    def set_value(self, value):
+        """Save the plain value."""
+        self.plain_value = self.to_plain(value)
+        self.encrypted_value.cipher_value = None
 
     def check(self):
         """Check whether the embedded MAC is correct."""
@@ -70,29 +72,32 @@ class DataType(object):
 class BinaryDataType(DataType):
     """Subclass of DataType for binary data (e.g. keys)."""
 
-    @property
-    def value(self):
+    def get_value(self):
         """Provide the raw binary value."""
         # plain value is base64 encoded
-        value = self.plain_value
-        if value is not None:
-            return base64.b64decode(value)
+        if self.plain_value is not None:
+            return base64.b64decode(self.plain_value)
+        # check MAC if present
+        self.check()
         # encrypted value is in correct format
-        value = self.encrypted_value.decrypt()
-        if value is not None:
-            return value
+        return self.encrypted_value.decrypt()
+
+    def to_plain(self, value):
+        """Convert the value to an unencrypted string representation."""
+        if value:
+            return base64.b64encode(value)
 
 
 class IntegerDataType(DataType):
     """Subclass of DataType for integer types (e.g. counters)."""
 
-    @property
-    def value(self):
+    def get_value(self):
         """Provide the raw integer value."""
         # plain value is a string representation of the number
-        value = self.plain_value
-        if value:
-            return int(value)
+        if self.plain_value:
+            return int(self.plain_value)
+        # check MAC if present
+        self.check()
         # decrypted value is big endian encoded
         value = self.encrypted_value.decrypt()
         if value is not None:
@@ -102,6 +107,11 @@ class IntegerDataType(DataType):
                 v = (v << 8) + ord(x)
             return v
 
+    def to_plain(self, value):
+        """Convert the value to an unencrypted string representation."""
+        if value not in (None, ''):
+            return str(value)
+
 
 class Key(object):
     """Representation of a single key from a PSKC file.
@@ -140,7 +150,7 @@ class Key(object):
       policy: reference to policy information (see Policy class)
     """
 
-    def __init__(self, pskc, key_package):
+    def __init__(self, pskc, key_package=None):
 
         self.pskc = pskc
 
@@ -187,113 +197,99 @@ class Key(object):
 
     def parse(self, key_package):
         """Read key information from the provided <KeyPackage> tree."""
-        from pskc.parse import g_e_v, g_e_d, namespaces
+        from pskc.parse import find, findtext, findtime, getint, getbool
         if key_package is None:
             return
 
-        key = key_package.find('pskc:Key', namespaces=namespaces)
+        key = find(key_package, 'pskc:Key')
         if key is not None:
-            self.id = key.attrib.get('Id')
-            self.algorithm = key.attrib.get('Algorithm')
+            self.id = key.get('Id')
+            self.algorithm = key.get('Algorithm')
 
-        data = key_package.find('pskc:Key/pskc:Data', namespaces=namespaces)
+        data = find(key_package, 'pskc:Key/pskc:Data')
         if data is not None:
-            self._secret.parse(data.find(
-                'pskc:Secret', namespaces=namespaces))
-            self._counter.parse(data.find(
-                'pskc:Counter', namespaces=namespaces))
-            self._time_offset.parse(data.find(
-                'pskc:Time', namespaces=namespaces))
-            self._time_interval.parse(data.find(
-                'pskc:TimeInterval', namespaces=namespaces))
-            self._time_drift.parse(data.find(
-                'pskc:TimeDrift', namespaces=namespaces))
+            self._secret.parse(find(data, 'pskc:Secret'))
+            self._counter.parse(find(data, 'pskc:Counter'))
+            self._time_offset.parse(find(data, 'pskc:Time'))
+            self._time_interval.parse(find(data, 'pskc:TimeInterval'))
+            self._time_drift.parse(find(data, 'pskc:TimeDrift'))
 
-        self.issuer = g_e_v(key_package, 'pskc:Key/pskc:Issuer')
-        self.key_profile = g_e_v(key_package, 'pskc:Key/pskc:KeyProfileId')
-        self.key_reference = g_e_v(key_package, 'pskc:Key/pskc:KeyReference')
-        self.friendly_name = g_e_v(key_package, 'pskc:Key/pskc:FriendlyName')
+        self.issuer = findtext(key_package, 'pskc:Key/pskc:Issuer')
+        self.key_profile = findtext(key_package, 'pskc:Key/pskc:KeyProfileId')
+        self.key_reference = findtext(
+            key_package, 'pskc:Key/pskc:KeyReference')
+        self.friendly_name = findtext(
+            key_package, 'pskc:Key/pskc:FriendlyName')
         # TODO: support multi-language values of <FriendlyName>
-        self.key_userid = g_e_v(key_package, 'pskc:Key/pskc:UserId')
+        self.key_userid = findtext(key_package, 'pskc:Key/pskc:UserId')
 
-        self.manufacturer = g_e_v(
+        self.manufacturer = findtext(
             key_package, 'pskc:DeviceInfo/pskc:Manufacturer')
-        self.serial = g_e_v(key_package, 'pskc:DeviceInfo/pskc:SerialNo')
-        self.model = g_e_v(key_package, 'pskc:DeviceInfo/pskc:Model')
-        self.issue_no = g_e_v(key_package, 'pskc:DeviceInfo/pskc:IssueNo')
-        self.device_binding = g_e_v(
+        self.serial = findtext(key_package, 'pskc:DeviceInfo/pskc:SerialNo')
+        self.model = findtext(key_package, 'pskc:DeviceInfo/pskc:Model')
+        self.issue_no = findtext(key_package, 'pskc:DeviceInfo/pskc:IssueNo')
+        self.device_binding = findtext(
             key_package, 'pskc:DeviceInfo/pskc:DeviceBinding')
-        self.start_date = g_e_d(key_package, 'pskc:DeviceInfo/pskc:StartDate')
-        self.expiry_date = g_e_d(
+        self.start_date = findtime(
+            key_package, 'pskc:DeviceInfo/pskc:StartDate')
+        self.expiry_date = findtime(
             key_package, 'pskc:DeviceInfo/pskc:ExpiryDate')
-        self.device_userid = g_e_v(key_package, 'pskc:DeviceInfo/pskc:UserId')
+        self.device_userid = findtext(
+            key_package, 'pskc:DeviceInfo/pskc:UserId')
 
-        self.crypto_module = g_e_v(
+        self.crypto_module = findtext(
             key_package, 'pskc:CryptoModuleInfo/pskc:Id')
 
-        self.algorithm_suite = g_e_v(
+        self.algorithm_suite = findtext(
             key_package, 'pskc:Key/pskc:AlgorithmParameters/pskc:Suite')
 
-        challenge_format = key_package.find(
-            'pskc:Key/pskc:AlgorithmParameters/pskc:ChallengeFormat',
-            namespaces=namespaces)
+        challenge_format = find(
+            key_package,
+            'pskc:Key/pskc:AlgorithmParameters/pskc:ChallengeFormat')
         if challenge_format is not None:
-            self.challenge_encoding = challenge_format.attrib.get('Encoding')
-            v = challenge_format.attrib.get('Min')
-            if v:
-                self.challenge_min_length = int(v)
-            v = challenge_format.attrib.get('Max')
-            if v:
-                self.challenge_max_length = int(v)
-            v = challenge_format.attrib.get('CheckDigits')
-            if v:
-                self.challenge_check = v.lower() == 'true'
+            self.challenge_encoding = challenge_format.get('Encoding')
+            self.challenge_min_length = getint(challenge_format, 'Min')
+            self.challenge_max_length = getint(challenge_format, 'Max')
+            self.challenge_check = getbool(challenge_format, 'CheckDigits')
 
-        response_format = key_package.find(
-            'pskc:Key/pskc:AlgorithmParameters/pskc:ResponseFormat',
-            namespaces=namespaces)
+        response_format = find(
+            key_package,
+            'pskc:Key/pskc:AlgorithmParameters/pskc:ResponseFormat')
         if response_format is not None:
-            self.response_encoding = response_format.attrib.get('Encoding')
-            v = response_format.attrib.get('Length')
-            if v:
-                self.response_length = int(v)
-            v = response_format.attrib.get('CheckDigits')
-            if v:
-                self.response_check = v.lower() == 'true'
+            self.response_encoding = response_format.get('Encoding')
+            self.response_length = getint(response_format, 'Length')
+            self.response_check = getbool(response_format, 'CheckDigits')
 
-        self.policy.parse(key_package.find(
-            'pskc:Key/pskc:Policy', namespaces=namespaces))
+        self.policy.parse(find(key_package, 'pskc:Key/pskc:Policy'))
 
-    @property
-    def secret(self):
-        """The secret key itself."""
-        return self._secret.value
+    secret = property(
+        fget=lambda self: self._secret.get_value(),
+        fset=lambda self, x: self._secret.set_value(x),
+        doc="The secret key itself.")
 
-    @property
-    def counter(self):
-        """An event counter for event-based OTP."""
-        return self._counter.value
+    counter = property(
+        fget=lambda self: self._counter.get_value(),
+        fset=lambda self, x: self._counter.set_value(x),
+        doc="An event counter for event-based OTP.")
 
-    @property
-    def time_offset(self):
-        """A time offset for time-based OTP (number of intervals)."""
-        return self._time_offset.value
+    time_offset = property(
+        fget=lambda self: self._time_offset.get_value(),
+        fset=lambda self, x: self._time_offset.set_value(x),
+        doc="A time offset for time-based OTP (number of intervals).")
 
-    @property
-    def time_interval(self):
-        """A time interval in seconds."""
-        return self._time_interval.value
+    time_interval = property(
+        fget=lambda self: self._time_interval.get_value(),
+        fset=lambda self, x: self._time_interval.set_value(x),
+        doc="A time interval in seconds.")
 
-    @property
-    def time_drift(self):
-        """Device clock drift value (number of time intervals)."""
-        return self._time_drift.value
+    time_drift = property(
+        fget=lambda self: self._time_drift.get_value(),
+        fset=lambda self, x: self._time_drift.set_value(x),
+        doc="Device clock drift value (number of time intervals).")
 
     def check(self):
         """Check if all MACs in the message are valid."""
-        checks = (self._secret.check(), self._counter.check(),
-                  self._time_offset.check(), self._time_interval.check(),
-                  self._time_drift.check())
-        if all(x is None for x in checks):
-            return None
-        return all(x is not False for x in checks)
+        if any((self._secret.check(), self._counter.check(),
+                self._time_offset.check(), self._time_interval.check(),
+                self._time_drift.check())):
+            return True
