@@ -29,14 +29,22 @@ with the PSKC encryption key.
 """
 
 
-import base64
-import hashlib
-import hmac
-
-from pskc.encryption import EncryptedValue
+import re
 
 
-HMAC_SHA1 = 'http://www.w3.org/2000/09/xmldsig#hmac-sha1'
+_hmac_url_re = re.compile(r'^.*#hmac-(?P<hash>[a-z0-9]+)$')
+
+
+def get_hmac(algorithm):
+    """Return an HMAC function that takes a secret and a value and returns a
+    digest."""
+    import hashlib
+    import hmac
+    match = _hmac_url_re.search(algorithm)
+    if match:
+        digestmod = getattr(hashlib, match.group('hash'), None)
+        if digestmod is not None:
+            return lambda key, value: hmac.new(key, value, digestmod).digest()
 
 
 class ValueMAC(object):
@@ -49,26 +57,30 @@ class ValueMAC(object):
 
     def parse(self, value_mac):
         """Read MAC information from the <ValueMAC> XML tree."""
-        from pskc.parse import g_e_v
+        from pskc.parse import findbin
         if value_mac is None:
             return
-        value = g_e_v(value_mac, '.')
-        if value is not None:
-            self._value_mac = base64.b64decode(value)
+        self._value_mac = findbin(value_mac, '.')
 
     def check(self, value):
         """Check if the provided value matches the MAC.
 
-        This will return None if the value cannot be checked (no value,
-        no key, etc.) or a boolean otherwise.
+        This will return None if there is no MAC to be checked. It will
+        return True if the MAC matches and raise an exception if it fails.
         """
+        from pskc.exceptions import DecryptionError
         if value is None or self._value_mac is None:
-            return
-        algorithm = self.mac.algorithm
+            return  # no MAC present or nothing to check
         key = self.mac.key
-        if algorithm == HMAC_SHA1 and key is not None:
-            h = hmac.new(key, value, hashlib.sha1).digest()
-            return h == self._value_mac
+        if key is None:
+            raise DecryptionError('No MAC key available')
+        hmacfn = get_hmac(self.mac.algorithm)
+        if hmacfn is None:
+            raise DecryptionError(
+                'Unsupported MAC algorithm: %r' % self.mac.algorithm)
+        if hmacfn(key, value) != self._value_mac:
+            raise DecryptionError('MAC value does not match')
+        return True
 
 
 class MAC(object):
@@ -81,19 +93,19 @@ class MAC(object):
     """
 
     def __init__(self, pskc, mac_method=None):
+        from pskc.encryption import EncryptedValue
         self.algorithm = None
         self._mac_key = EncryptedValue(pskc.encryption)
         self.parse(mac_method)
 
     def parse(self, mac_method):
         """Read MAC information from the <MACMethod> XML tree."""
-        from pskc.parse import g_e_v, namespaces
+        from pskc.parse import find, findtext
         if mac_method is None:
             return
-        self.algorithm = mac_method.attrib.get('Algorithm')
-        self._mac_key.parse(mac_method.find(
-            'pskc:MACKey', namespaces=namespaces))
-        mac_key_reference = g_e_v(mac_method, 'pskc:MACKeyReference')
+        self.algorithm = mac_method.get('Algorithm')
+        self._mac_key.parse(find(mac_method, 'pskc:MACKey'))
+        mac_key_reference = findtext(mac_method, 'pskc:MACKeyReference')
 
     @property
     def key(self):
